@@ -1,5 +1,21 @@
 """
-Main Classes for the PeakRDL Python
+peakrdl-python is a tool to generate Python Register Access Layer (RAL) from SystemRDL
+Copyright (C) 2021 - 2023
+
+This program is free software: you can redistribute it and/or modify
+it under the terms of the GNU General Public License as published by
+the Free Software Foundation, either version 3 of the License, or
+(at your option) any later version.
+
+This program is distributed in the hope that it will be useful,
+but WITHOUT ANY WARRANTY; without even the implied warranty of
+MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+GNU General Public License for more details.
+
+You should have received a copy of the GNU General Public License
+along with this program.  If not, see <https://www.gnu.org/licenses/>.
+
+Main Classes for the peakrdl-python
 """
 import os
 from pathlib import Path
@@ -235,6 +251,8 @@ class PythonExporter:
         else:
             top_block = node
 
+        if not isinstance(path, str):
+            raise TypeError(f'path should be a str but got {type(path)}')
         package = _Package(path=path,
                            package_name=node.inst_name,
                            include_tests=not skip_test_case_generation)
@@ -323,6 +341,19 @@ class PythonExporter:
                 # top block, including the top_block itself
                 RDLWalker(unroll=True).walk(block, owned_elements, skip_top=True)
 
+                # The code that generates the tests for the register array context managers needs
+                # the arrays rolled up but parents within the address map e.g. a regfile unrolled
+                # I have not found a way to do this with the Walker as the unroll seems to be a
+                # global setting, the following code works but it is not elegant
+                rolled_owned_reg: List[RegNode] = list(block.registers(unroll=False))
+                for regfile in owned_elements.reg_files:
+                    rolled_owned_reg += list(regfile.registers(unroll=False))
+                for memory in owned_elements.memories:
+                    rolled_owned_reg += list(memory.registers(unroll=False))
+                def is_reg_array(item: RegNode) -> bool:
+                    return item.is_array
+                rolled_owned_reg_array = filter(is_reg_array, rolled_owned_reg)
+
                 fq_block_name = '_'.join(block.get_path_segments(array_suffix = '_{index:d}_'))
                 module_tb_path = package.tests.child_module_path('test_' + fq_block_name + '.py')
 
@@ -331,6 +362,7 @@ class PythonExporter:
                     'block' : block,
                     'fq_block_name' : fq_block_name,
                     'owned_elements': owned_elements,
+                    'rolled_owned_reg_array' : rolled_owned_reg_array,
                     'systemrdlFieldNode': FieldNode,
                     'systemrdlSignalNode': SignalNode,
                     'systemrdlRegNode': RegNode,
